// src/api/enums.ts
var ApplicationCommandType = {
  CHAT_INPUT: 1,
  USER: 2,
  MESSAGE: 3
};
var ApplicationCommandOptionType = {
  SUB_COMMAND: 1,
  SUB_COMMAND_GROUP: 2,
  STRING: 3,
  INTEGER: 4,
  BOOLEAN: 5,
  USER: 6,
  CHANNEL: 7,
  ROLE: 8,
  MENTIONABLE: 9,
  NUMBER: 10
};
var InteractionType = {
  PING: 1,
  APPLICATION_COMMAND: 2,
  MESSAGE_COMPONENT: 3,
  APPLICATION_COMMAND_AUTOCOMPLETE: 4
};
var InteractionResponseType = {
  PONG: 1,
  CHANNEL_MESSAGE_WITH_SOURCE: 4,
  DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE: 5,
  DEFERRED_MESSAGE_UPDATE: 6,
  UPDATE_MESSAGE: 7,
  APPLICATION_COMMAND_AUTOCOMPLETE_RESULT: 8
};
var ChannelType = {
  GUILD_TEXT: 0,
  DM: 1,
  GUILD_VOICE: 2,
  GROUP_DM: 3,
  GUILD_CATEGORY: 4,
  GUILD_NEWS: 5,
  GUILD_STORE: 6,
  GUILD_NEWS_THREAD: 10,
  GUILD_PUBLIC_THREAD: 11,
  GUILD_PRIVATE_THREAD: 12,
  GUILD_STAGE_VOICE: 13
};
var ComponentType = {
  ACTION_ROW: 1,
  BUTTON: 2,
  SELECT_MENU: 3
};
var ButtonStyle = {
  PRIMARY: 1,
  SECONDARY: 2,
  SUCCESS: 3,
  DANGER: 4,
  LINK: 5
};
var AllowedMentionsTypes = {
  ROLES: "roles",
  USERS: "users",
  EVERYONE: "everyone"
};

// src/helpers.ts
var PREFIX = "slshx";
function isGenerator(value) {
  return typeof value?.next === "function" && typeof value?.throw === "function" && typeof value?.return === "function";
}
function hexDecode(data) {
  return new Uint8Array(data.match(/.{2}/g)?.map((byte) => parseInt(byte, 16)) ?? []);
}
function extractAttachments(message) {
  if (!message.attachments?.length)
    return [message, void 0];
  const formData = new FormData();
  const attachments = [];
  for (let i = 0; i < message.attachments.length; i++) {
    formData.set(`files[${i}]`, message.attachments[i]);
    attachments.push({
      id: i,
      filename: message.attachments[i].name
    });
  }
  return [{ ...message, attachments }, formData];
}
function mergeFormData(body2, formData) {
  if (formData) {
    formData.set("payload_json", JSON.stringify(body2));
    return formData;
  } else {
    return body2;
  }
}
function jsonResponse(body2, formData) {
  if (formData) {
    formData.set("payload_json", JSON.stringify(body2));
    return new Response(formData);
  } else {
    return new Response(JSON.stringify(body2), {
      headers: { "Content-Type": "application/json" }
    });
  }
}
function log(...data) {
  console.log(`[35m[${PREFIX}]`, ...data, "[39m");
}

// src/api/helpers.ts
var APIError = class extends Error {
  constructor(method, path, code, message) {
    super(message);
    this.method = method;
    this.path = path;
    this.code = code;
    this.name = `APIError [${code}]`;
  }
};
var RELATIVE_FORMAT = /* @__PURE__ */ new Intl.RelativeTimeFormat("en", {
  numeric: "always",
  style: "long"
});
async function call(method, path, body2, auth) {
  const url = `https://discord.com/api/v9${path}`;
  const headers = {};
  const init = { method, headers };
  if (body2 instanceof FormData) {
    init.body = body2;
  } else if (body2 instanceof URLSearchParams) {
    headers["Content-Type"] = "application/x-www-form-urlencoded";
    init.body = body2.toString();
  } else if (body2) {
    headers["Content-Type"] = "application/json";
    init.body = JSON.stringify(body2);
  }
  if (auth) {
    headers["Authorization"] = "bearer" in auth ? `Bearer ${auth.bearer}` : `Basic ${btoa(`${auth.username}:${auth.password}`)}`;
  }
  const res = await fetch(url, init);
  if (globalThis.MINIFLARE) {
    let resetString = "";
    const limit = res.headers.get("x-ratelimit-limit");
    const remaining = res.headers.get("x-ratelimit-remaining");
    const reset = res.headers.get("x-ratelimit-reset");
    if (limit && remaining && reset) {
      resetString = `(${remaining}/${limit} remaining, reset ${RELATIVE_FORMAT.format(Math.round(parseFloat(reset) - Date.now() / 1e3), "second")})`;
    }
    path = path.replace(/[a-z0-9]{50,}/i, "***") + ":";
    log(method, path, res.status.toString(), res.statusText, resetString);
  }
  if (res.status === 204) {
    return void 0;
  } else if (res.ok) {
    return await res.json();
  }
  let error = await res.text();
  try {
    error = JSON.stringify(JSON.parse(error), null, 2);
  } catch {
  }
  throw new APIError(method, path, res.status, error);
}
async function getBearerAuth(applicationId, applicationSecret) {
  const body2 = new URLSearchParams({
    grant_type: "client_credentials",
    scope: "applications.commands.update"
  });
  const auth = {
    username: applicationId,
    password: applicationSecret
  };
  const res = await call("POST", "/oauth2/token", body2, auth);
  return {
    bearer: res.access_token,
    expires: Date.now() + res.expires_in * 1e3
  };
}

// src/api/commands.ts
function getGlobalApplicationCommands(applicationId, auth) {
  return call("GET", `/applications/${applicationId}/commands`, 0, auth);
}
function createGlobalApplicationCommand(applicationId, command, auth) {
  return call("POST", `/applications/${applicationId}/commands`, command, auth);
}
function getGlobalApplicationCommand(applicationId, commandId, auth) {
  return call("GET", `/applications/${applicationId}/commands/${commandId}`, 0, auth);
}
function editGlobalApplicationCommand(applicationId, commandId, command, auth) {
  return call("PATCH", `/applications/${applicationId}/commands/${commandId}`, command, auth);
}
function deleteGlobalApplicationCommand(applicationId, commandId, auth) {
  return call("DELETE", `/applications/${applicationId}/commands/${commandId}`, 0, auth);
}
function bulkOverwriteGlobalApplicationCommands(applicationId, commands, auth) {
  return call("PUT", `/applications/${applicationId}/commands`, commands, auth);
}
function getGuildApplicationCommands(applicationId, guildId, auth) {
  return call("GET", `/applications/${applicationId}/guilds/${guildId}/commands`, 0, auth);
}
function createGuildApplicationCommand(applicationId, guildId, command, auth) {
  return call("POST", `/applications/${applicationId}/guilds/${guildId}/commands`, command, auth);
}
function getGuildApplicationCommand(applicationId, guildId, commandId, auth) {
  return call("GET", `/applications/${applicationId}/guilds/${guildId}/commands/${commandId}`, 0, auth);
}
function editGuildApplicationCommand(applicationId, guildId, commandId, command, auth) {
  return call("PATCH", `/applications/${applicationId}/guilds/${guildId}/commands/${commandId}`, command, auth);
}
function deleteGuildApplicationCommand(applicationId, guildId, commandId, auth) {
  return call("DELETE", `/applications/${applicationId}/guilds/${guildId}/commands/${commandId}`, 0, auth);
}
function bulkOverwriteGuildApplicationCommands(applicationId, guildId, commands, auth) {
  return call("PUT", `/applications/${applicationId}/guilds/${guildId}/commands`, commands, auth);
}

// src/api/interactions.ts
function getOriginalInteractionResponse(applicationId, interactionToken) {
  return call("GET", `/webhooks/${applicationId}/${interactionToken}/messages/@original`);
}
function editOriginalInteractionResponse(applicationId, interactionToken, message) {
  const body2 = mergeFormData(...extractAttachments(message));
  return call("PATCH", `/webhooks/${applicationId}/${interactionToken}/messages/@original`, body2);
}
function deleteOriginalInteractionResponse(applicationId, interactionToken) {
  return call("DELETE", `/webhooks/${applicationId}/${interactionToken}/messages/@original`);
}
function createFollowupMessage(applicationId, interactionToken, message) {
  const body2 = mergeFormData(...extractAttachments(message));
  return call("POST", `/webhooks/${applicationId}/${interactionToken}`, body2);
}
function getFollowupMessage(applicationId, interactionToken, messageId) {
  return call("GET", `/webhooks/${applicationId}/${interactionToken}/messages/${messageId}`);
}
function editFollowupMessage(applicationId, interactionToken, messageId, message) {
  const body2 = mergeFormData(...extractAttachments(message));
  return call("PATCH", `/webhooks/${applicationId}/${interactionToken}/messages/${messageId}`, body2);
}
function deleteFollowupMessage(applicationId, interactionToken, messageId) {
  return call("DELETE", `/webhooks/${applicationId}/${interactionToken}/messages/${messageId}`);
}

// src/api/permissions.ts
function getGuildApplicationCommandPermissions(applicationId, guildId, auth) {
  return call("GET", `/applications/${applicationId}/guilds/${guildId}/commands/permissions`, 0, auth);
}
function getApplicationCommandPermissions(applicationId, guildId, commandId, auth) {
  return call("GET", `/applications/${applicationId}/guilds/${guildId}/commands/${commandId}/permissions`, 0, auth);
}
function editApplicationCommandPermissions(applicationId, guildId, commandId, permissions, auth) {
  return call("PUT", `/applications/${applicationId}/guilds/${guildId}/commands/${commandId}/permissions`, permissions, auth);
}
function bulkEditApplicationCommandPermissions(applicationId, guildId, permissions, auth) {
  return call("PUT", `/applications/${applicationId}/guilds/${guildId}/commands/permissions`, permissions, auth);
}

// src/commands/state.ts
var STATE = {
  recordingDescription: "",
  componentHandlerCount: 0
};

// src/commands/recorders.ts
function validateName(name) {
  if (/[:/$#]/.test(name)) {
    throw new RangeError(`Command name "${name}" must not contain ':', '/', '$' or '#' characters`);
  }
}
function recordCommand(commandId, name, command) {
  STATE.commandId = commandId;
  STATE.recordingOptions = [];
  STATE.recordingDescription = "";
  STATE.recordingDefaultPermission = void 0;
  STATE.componentHandlerCount = 0;
  try {
    command();
    return {
      name,
      description: STATE.recordingDescription,
      options: STATE.recordingOptions.length ? STATE.recordingOptions : void 0,
      default_permission: STATE.recordingDefaultPermission
    };
  } finally {
    STATE.commandId = void 0;
    STATE.recordingOptions = void 0;
  }
}
function recordChatInputCommands(commands) {
  const results = [];
  const type = ApplicationCommandType.CHAT_INPUT;
  for (const [name, command] of Object.entries(commands)) {
    validateName(name);
    if (typeof command === "function") {
      const id = `${PREFIX}:${type}/${name}`;
      results.push(recordCommand(id, name, command));
    } else {
      let defaultPermission = true;
      const commandOptions = [];
      for (const [subGroupName, subGroupCommand] of Object.entries(command)) {
        validateName(subGroupName);
        if (typeof subGroupCommand === "function") {
          const id = `${PREFIX}:${type}/${name}/${subGroupName}`;
          const cmd = recordCommand(id, subGroupName, subGroupCommand);
          if (cmd.default_permission === false)
            defaultPermission = false;
          delete cmd.default_permission;
          commandOptions.push({
            type: ApplicationCommandOptionType.SUB_COMMAND,
            ...cmd
          });
        } else {
          const subGroupCommandOptions = [];
          for (const [subCommandName, subCommand] of Object.entries(subGroupCommand)) {
            validateName(subCommandName);
            const id = `${PREFIX}:${type}/${name}/${subGroupName}/${subCommandName}`;
            const cmd = recordCommand(id, subCommandName, subCommand);
            if (cmd.default_permission === false)
              defaultPermission = false;
            delete cmd.default_permission;
            subGroupCommandOptions.push({
              type: ApplicationCommandOptionType.SUB_COMMAND,
              ...cmd
            });
          }
          commandOptions.push({
            type: ApplicationCommandOptionType.SUB_COMMAND_GROUP,
            name: subGroupName,
            description: subGroupName,
            options: subGroupCommandOptions
          });
        }
      }
      results.push({
        name,
        description: name,
        options: commandOptions,
        default_permission: defaultPermission ? void 0 : false
      });
    }
  }
  return results;
}
function recordContextMenuCommands(type, commands) {
  const results = [];
  for (const [name, command] of Object.entries(commands)) {
    validateName(name);
    const id = `${PREFIX}:${type}/${name}`;
    results.push({ type, ...recordCommand(id, name, command) });
  }
  return results;
}
function recordCommands(opts) {
  const commands = [];
  if (opts.commands) {
    commands.push(...recordChatInputCommands(opts.commands));
  }
  if (opts.userCommands) {
    commands.push(...recordContextMenuCommands(ApplicationCommandType.USER, opts.userCommands));
  }
  if (opts.messageCommands) {
    commands.push(...recordContextMenuCommands(ApplicationCommandType.MESSAGE, opts.messageCommands));
  }
  return commands;
}

// src/commands/deploy.ts
var CACHE = globalThis.MINIFLARE ? BigInt64Array.$SLSHXCACHE ??= {} : {};
async function deployCommands(opts) {
  if (!opts.applicationId || !opts.applicationSecret) {
    throw new Error("Deploying requires applicationId and applicationSecret to be set");
  }
  log("Building and deploying commands...");
  const commands = recordCommands(opts);
  if (!CACHE.bearerAuth?.expires || CACHE.bearerAuth.expires - 3e4 < Date.now()) {
    log("Refreshing access token...");
    CACHE.bearerAuth = await getBearerAuth(opts.applicationId, opts.applicationSecret);
  }
  const s = commands.length === 1 ? "" : "s";
  if (opts.testServerId) {
    const jsonCommands = JSON.stringify(commands);
    if (CACHE.jsonCommands === jsonCommands) {
      log(`No changes since last deploy!`);
      return;
    }
    log(`Deploying ${commands.length} command${s} to server ${opts.testServerId}...`);
    await bulkOverwriteGuildApplicationCommands(opts.applicationId, opts.testServerId, commands, CACHE.bearerAuth);
    log(`Deployed ${commands.length} command${s} to server ${opts.testServerId}!`);
    CACHE.jsonCommands = jsonCommands;
  } else {
    log(`Deploying ${commands.length} command${s} globally...`);
    await bulkOverwriteGlobalApplicationCommands(opts.applicationId, commands, CACHE.bearerAuth);
    log(`Deployed ${commands.length} command${s} globally!`);
  }
}

// src/commands/handlers.ts
function instantiateCommandHandler(commandId, command, options, resolved) {
  STATE.commandId = commandId;
  STATE.interactionOptions = options;
  STATE.interactionResolved = resolved;
  STATE.componentHandlerCount = 0;
  try {
    return command();
  } finally {
    STATE.commandId = void 0;
    STATE.interactionOptions = void 0;
    STATE.interactionResolved = void 0;
  }
}
function instantiateComponentHandler(commandId, command, customId) {
  STATE.commandId = commandId;
  STATE.componentHandlers = new Map();
  STATE.componentHandlerCount = 0;
  try {
    command();
    customId = customId.substring(0, customId.indexOf("#") + 1);
    return STATE.componentHandlers.get(customId);
  } finally {
    STATE.commandId = void 0;
    STATE.componentHandlers = void 0;
  }
}
function instantiateAutocompleteHandler(commandId, command, options) {
  STATE.commandId = commandId;
  STATE.interactionOptions = options;
  STATE.componentHandlerCount = 0;
  STATE.autocompleteHandlers = new Map();
  try {
    command();
    if (!options)
      return;
    for (const [name, option] of options) {
      if (option.focused)
        return STATE.autocompleteHandlers.get(name);
    }
  } finally {
    STATE.commandId = void 0;
    STATE.interactionOptions = void 0;
    STATE.autocompleteHandlers = void 0;
  }
}

// src/commands/hooks.ts
function useDescription(description) {
  if (!STATE.commandId) {
    throw new Error(`Hooks must be called inside a command`);
  }
  if (STATE.recordingOptions) {
    STATE.recordingDescription = description;
  }
}
function useDefaultPermission(permission) {
  if (!STATE.commandId) {
    throw new Error(`Hooks must be called inside a command`);
  }
  if (STATE.recordingOptions) {
    STATE.recordingDefaultPermission = permission;
  }
}
function useComponent(handler) {
  if (!STATE.commandId) {
    throw new Error(`Hooks must be called inside a command`);
  }
  const customId = `${STATE.commandId}$${STATE.componentHandlerCount++}#`;
  if (STATE.componentHandlers) {
    STATE.componentHandlers.set(customId, handler);
  }
  return customId;
}
function useButton(handler) {
  return useComponent(handler);
}
function useSelectMenu(handler) {
  return useComponent(handler);
}
function normaliseChoice(choice) {
  if (typeof choice === "object" && "value" in choice) {
    return { name: choice.name ?? String(choice.value), value: choice.value };
  }
  const name = String(choice);
  return { name, value: typeof choice === "number" ? choice : name };
}
function normaliseChoices(choices) {
  return choices?.map(normaliseChoice);
}
function useOption(type, empty, name, description, options) {
  if (!STATE.commandId) {
    throw new Error(`Hooks must be called inside a command`);
  }
  if (STATE.autocompleteHandlers && options?.autocomplete) {
    STATE.autocompleteHandlers.set(name, options.autocomplete);
  }
  const def = options?.required ? options.choices?.length ? normaliseChoice(options.choices[0]).value : empty : null;
  if (STATE.interactionOptions) {
    return STATE.interactionOptions.get(name)?.value ?? def;
  }
  if (STATE.recordingOptions) {
    STATE.recordingOptions.push({
      type,
      name,
      description,
      required: options?.required,
      autocomplete: options?.autocomplete && true,
      choices: normaliseChoices(options?.choices),
      channel_types: options?.types,
      min_value: options?.min,
      max_value: options?.max
    });
  }
  return def;
}
function useString(name, description, options) {
  return useOption(ApplicationCommandOptionType.STRING, "", name, description, options);
}
function useInteger(name, description, options) {
  return useOption(ApplicationCommandOptionType.INTEGER, 0, name, description, options);
}
function useBoolean(name, description, options) {
  return useOption(ApplicationCommandOptionType.BOOLEAN, false, name, description, options);
}
function useUser(name, description, options) {
  const id = useOption(ApplicationCommandOptionType.USER, "", name, description, options);
  return STATE.interactionResolved?.users?.[id] ?? null;
}
function useChannel(name, description, options) {
  const id = useOption(ApplicationCommandOptionType.CHANNEL, "", name, description, options);
  return STATE.interactionResolved?.channels?.[id] ?? null;
}
function useRole(name, description, options) {
  const id = useOption(ApplicationCommandOptionType.ROLE, "", name, description, options);
  return STATE.interactionResolved?.roles?.[id] ?? null;
}
function useMentionable(name, description, options) {
  const id = useOption(ApplicationCommandOptionType.MENTIONABLE, "", name, description, options);
  return STATE.interactionResolved?.users?.[id] ?? STATE.interactionResolved?.roles?.[id] ?? null;
}
function useNumber(name, description, options) {
  return useOption(ApplicationCommandOptionType.NUMBER, 0, name, description, options);
}

// src/commands/types.ts
var $update = /* @__PURE__ */ Symbol("$update");

// src/interactions/validate.ts
var ENCODER = /* @__PURE__ */ new TextEncoder();
async function validateInteraction(publicKeyData, request) {
  const signature = hexDecode(String(request.headers.get("X-Signature-Ed25519")));
  const timestamp = String(request.headers.get("X-Signature-Timestamp"));
  const body2 = await request.text();
  const publicKey = await crypto.subtle.importKey("raw", publicKeyData, { name: "NODE-ED25519", namedCurve: "NODE-ED25519", public: true }, true, ["verify"]);
  const valid = await crypto.subtle.verify("NODE-ED25519", publicKey, signature, ENCODER.encode(timestamp + body2));
  return valid && JSON.parse(body2);
}

// src/interactions/ping.ts
function handlePingInteraction() {
  return jsonResponse({
    type: InteractionResponseType.PONG
  });
}

// src/interactions/command.ts
function matchCommand(interaction, opts) {
  if (!interaction.data)
    return ["", void 0, void 0];
  const type = interaction.data.type;
  let data = interaction.data;
  let id = `${PREFIX}:${type}/${data.name}`;
  let command;
  if (type === ApplicationCommandType.CHAT_INPUT) {
    let cursor;
    let group = opts.commands;
    while (cursor = group?.[data.name]) {
      if (typeof cursor === "function") {
        command = cursor;
        break;
      }
      group = cursor;
      data = data.options[0];
      id += `/${data.name}`;
    }
  } else {
    command = (type === ApplicationCommandType.USER ? opts.userCommands : opts.messageCommands)?.[data.name];
  }
  let options;
  if (command && data.options) {
    options = new Map();
    for (const option of data.options) {
      options.set(option.name, option);
    }
  }
  return [id, command, options];
}
async function handleApplicationCommandInteraction(interaction, opts, env, ctx) {
  const [commandId, command, options] = matchCommand(interaction, opts);
  if (!command) {
    return new Response("Command Not Found", { status: 404 });
  }
  const resolved = interaction.data.resolved;
  const handler = instantiateCommandHandler(commandId, command, options, resolved);
  let target;
  if ("target_id" in interaction.data) {
    if (interaction.data.type === ApplicationCommandType.USER) {
      target = interaction.data.resolved.users[interaction.data.target_id];
    } else if (interaction.data.type === ApplicationCommandType.MESSAGE) {
      target = interaction.data.resolved.messages[interaction.data.target_id];
    }
  }
  let res = await handler(interaction, env, ctx, target);
  if (isGenerator(res)) {
    const yieldRes = await res.next();
    if (!yieldRes.done) {
      const deferredPromise = Promise.resolve(res.next()).then((res2) => {
        if (!res2.done)
          throw new Error("Response can only be deferred once");
        return createFollowupMessage(opts.applicationId, interaction.token, res2.value);
      });
      ctx.waitUntil(deferredPromise);
      return jsonResponse({ type: InteractionResponseType.DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE });
    }
    res = yieldRes.value;
  }
  const [newBody, formData] = extractAttachments(res);
  return jsonResponse({
    type: InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
    data: newBody
  }, formData);
}

// src/interactions/component.ts
async function handleMessageComponentInteraction(interaction, opts, env, ctx) {
  const customId = interaction.data.custom_id;
  const commandId = customId.substring(0, customId.indexOf("$"));
  const locator = commandId.substring(PREFIX.length + 1);
  const parts = locator.split("/");
  const type = parseInt(parts.shift());
  let command;
  if (type === ApplicationCommandType.CHAT_INPUT) {
    command = opts.commands?.[parts.shift()];
    let part;
    while (typeof command === "object" && (part = parts.shift())) {
      command = command[part];
    }
  } else {
    command = (type === ApplicationCommandType.USER ? opts.userCommands : opts.messageCommands)?.[parts[0]];
  }
  if (typeof command !== "function") {
    return new Response("Command Not Found", { status: 404 });
  }
  const handler = instantiateComponentHandler(commandId, command, customId);
  if (!handler) {
    return new Response("Component Not Found", { status: 404 });
  }
  let res = await handler(interaction, env, ctx);
  if (isGenerator(res)) {
    const yieldRes = await res.next();
    if (!yieldRes.done) {
      const update = yieldRes.value === $update;
      const resType2 = update ? InteractionResponseType.DEFERRED_MESSAGE_UPDATE : InteractionResponseType.DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE;
      const deferredPromise = Promise.resolve(res.next()).then((res2) => {
        if (!res2.done)
          throw new Error("Response can only be deferred once");
        return (update ? editOriginalInteractionResponse : createFollowupMessage)(opts.applicationId, interaction.token, res2.value);
      });
      ctx.waitUntil(deferredPromise);
      return jsonResponse({ type: resType2 });
    }
    res = yieldRes.value;
  }
  const [newBody, formData] = extractAttachments(res);
  const resType = res[$update] ? InteractionResponseType.UPDATE_MESSAGE : InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE;
  return jsonResponse({ type: resType, data: newBody }, formData);
}

// src/interactions/autocomplete.ts
async function handleAutocompleteInteraction(interaction, opts, env, ctx) {
  const [commandId, command, options] = matchCommand(interaction, opts);
  if (!command) {
    return new Response("Command Not Found", { status: 404 });
  }
  const handler = instantiateAutocompleteHandler(commandId, command, options);
  if (!handler) {
    return new Response("Autocomplete Not Found", { status: 404 });
  }
  const choices = normaliseChoices(await handler(interaction, env, ctx));
  return jsonResponse({
    type: InteractionResponseType.APPLICATION_COMMAND_AUTOCOMPLETE_RESULT,
    data: { choices }
  });
}

// src/landing.ts
function authorizeResponse(applicationId) {
  const url = new URL("https://discord.com/api/oauth2/authorize");
  url.searchParams.set("client_id", applicationId);
  url.searchParams.set("scope", "applications.commands");
  return Response.redirect(url.toString());
}
var body = `<!doctype html>
<html lang="en">
<head>
<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">
<meta http-equiv="X-UA-Compatible" content="ie=edge">
<title>\u2694\uFE0F Slshx</title>
<style>
body {
  margin: 0;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 14px;
  text-align: center;
  background-color: #36393f;
  color: #fff;
}
h1 { margin: 16px 0; font-size: 128px }
p { line-height: 1.5 }
form { margin-bottom: 8px }
input[type=submit] {
  width: 100%;
  max-width: 250px;
  text-align: center;
  border: none;
  border-radius: 3px;
  height: 32px;
  cursor: pointer;
  color: #fff;
  background-color: #4f545b;
  transition: background-color .17s ease;
}
input[type=submit]:hover { background-color: #5d6269 }
input[type=submit]:active { background-color: #72767d }
input[type=submit].primary { background-color: #5865f2 }
input[type=submit].primary:hover { background-color: #4752c4 }
input[type=submit].primary:active { background-color: #3c45a5 }
</style>
</head>
<body>
<h1>\u2694\uFE0F</h1>
<form method="post" action="?slshx_action=authorize" target="_blank"><input type="submit" value="Add to Server" class="primary"></form>
<form method="post" action="?slshx_action=deploy"><input type="submit" value="Deploy Commands Globally"></form>
<p></p>
<script>
document.querySelectorAll("form")[1].onsubmit = function (e) {
  if(!confirm("\u26A0\uFE0F Are you sure you want to deploy globally? This will update the commands in EVERY server your app (%APP_ID%) has been added to. Changes may take up to an hour to propagate.\\n\\nNote that Slshx instantly deploys to just your test server whenever you change your code.")) e.preventDefault();
};
<\/script>
</body>
</html>
`;
async function handleLanding(opts, request) {
  const applicationId = opts.applicationId;
  if (!applicationId) {
    return Response.redirect("https://discord.com/developers/applications");
  }
  if (request.method === "GET") {
    return new Response(body.replace("%APP_ID%", applicationId), {
      headers: { "Content-Type": "text/html; charset=utf-8" }
    });
  } else if (request.method === "POST") {
    const url = new URL(request.url);
    const action = url.searchParams.get("slshx_action");
    if (action === "authorize")
      return authorizeResponse(applicationId);
    if (action === "deploy") {
      await deployCommands({ ...opts, testServerId: void 0 });
      return new Response(body.replace("%APP_ID%", applicationId).replace("<p></p>", "<p>Deployed! \u2705 <br><i>(changes may take up to an hour to propagate)</i></p>"), {
        headers: { "Content-Type": "text/html; charset=utf-8" }
      });
    }
  }
}

// src/jsx/helpers.ts
function isEmptyChild(child) {
  return !child && child !== 0 && child !== "" || child === true;
}
function childrenContent(children) {
  let content = void 0;
  for (const child of children?.flat(Infinity) ?? []) {
    if (isEmptyChild(child))
      continue;
    content ??= "";
    content += child;
  }
  return content;
}

// src/jsx/components/Row.ts
var $actionRow = /* @__PURE__ */ Symbol("$actionRow");
var $actionRowChild = /* @__PURE__ */ Symbol("$actionRowChild");
function Row(props) {
  return {
    [$actionRow]: true,
    type: ComponentType.ACTION_ROW,
    components: props.children?.flat(Infinity) ?? []
  };
}

// src/jsx/components/Button.ts
function Button(props) {
  return {
    [$actionRowChild]: true,
    type: ComponentType.BUTTON,
    ..."url" in props ? { url: props.url } : { custom_id: props.id },
    emoji: typeof props.emoji === "string" ? { name: props.emoji } : props.emoji,
    disabled: props.disabled,
    label: childrenContent(props.children),
    style: "url" in props ? ButtonStyle.LINK : props.primary ? ButtonStyle.PRIMARY : props.success ? ButtonStyle.SUCCESS : props.danger ? ButtonStyle.DANGER : ButtonStyle.SECONDARY
  };
}

// src/jsx/components/Option.ts
function Option(props) {
  return {
    value: props.value,
    description: props.description,
    emoji: typeof props.emoji === "string" ? { name: props.emoji } : props.emoji,
    default: props.default,
    label: childrenContent(props.children) ?? props.value
  };
}

// src/jsx/components/Select.ts
function Select(props) {
  return {
    [$actionRowChild]: true,
    type: ComponentType.SELECT_MENU,
    custom_id: props.id,
    placeholder: props.placeholder,
    min_values: props.min,
    max_values: props.max,
    disabled: props.disabled,
    options: props.children?.flat(Infinity) ?? []
  };
}

// src/jsx/embeds/Field.ts
var $field = /* @__PURE__ */ Symbol("$field");
function Field(props) {
  return {
    [$field]: true,
    name: props.name,
    value: childrenContent(props.children) ?? "",
    inline: props.inline
  };
}

// src/jsx/embeds/Embed.ts
var $embed = /* @__PURE__ */ Symbol("$embed");
function normaliseMedia(props) {
  if (!props)
    return;
  if (typeof props === "string")
    return { url: props };
  return {
    url: props.url,
    proxy_url: props.proxyUrl,
    width: props.width,
    height: props.height
  };
}
function normaliseFooter(props) {
  if (!props)
    return;
  if (typeof props === "string")
    return { text: props };
  return {
    text: props.text,
    icon_url: props.iconUrl,
    proxy_icon_url: props.proxyIconUrl
  };
}
function normaliseProvider(props) {
  return typeof props === "string" ? { name: props } : props;
}
function normaliseAuthor(props) {
  if (!props)
    return;
  if (typeof props === "string")
    return { name: props };
  return {
    name: props.name,
    url: props.url,
    icon_url: props.iconUrl,
    proxy_icon_url: props.proxyIconUrl
  };
}
function Embed(props) {
  let description = void 0;
  const fields = [];
  for (const child of props.children?.flat(Infinity) ?? []) {
    if (isEmptyChild(child))
      continue;
    if (child[$field]) {
      fields.push(child);
    } else {
      description ??= "";
      description += child;
    }
  }
  return {
    [$embed]: true,
    title: props.title,
    description,
    url: props.url,
    timestamp: props.timestamp ? typeof props.timestamp === "string" ? props.timestamp : props.timestamp.toISOString() : void 0,
    color: props.color,
    image: normaliseMedia(props.image),
    thumbnail: normaliseMedia(props.thumbnail),
    video: normaliseMedia(props.video),
    footer: normaliseFooter(props.footer),
    provider: normaliseProvider(props.provider),
    author: normaliseAuthor(props.author),
    fields
  };
}

// src/jsx/Message.ts
function Message(props) {
  let content = void 0;
  const embeds = [];
  const components = [];
  for (const child of props.children?.flat(Infinity) ?? []) {
    if (isEmptyChild(child))
      continue;
    if (child[$embed]) {
      embeds.push(child);
    } else if (child[$actionRow]) {
      components.push(child);
    } else if (child[$actionRowChild]) {
      components.push({
        type: ComponentType.ACTION_ROW,
        components: [child]
      });
    } else {
      content ??= "";
      content += child;
    }
  }
  return {
    [$update]: props.update,
    content,
    embeds,
    components,
    attachments: props.attachments,
    tts: props.tts,
    flags: props.ephemeral ? 64 : void 0,
    allowed_mentions: props.allowedMentions && {
      parse: props.allowedMentions.parse,
      roles: props.allowedMentions.roles,
      users: props.allowedMentions.users,
      replied_user: props.allowedMentions.repliedUser
    }
  };
}

// src/jsx/index.tsx
function createElement(component, attributes, ...children) {
  return component({ ...attributes, children });
}
function Fragment({
  children
}) {
  return children;
}

// src/index.ts
function createHandler(opts) {
  const publicKeyData = opts.applicationPublicKey && hexDecode(opts.applicationPublicKey);
  if (globalThis.MINIFLARE && opts.testServerId)
    void deployCommands(opts);
  return async function(request, env, ctx) {
    if (globalThis.MINIFLARE) {
      const res = await handleLanding(opts, request);
      if (res)
        return res;
    }
    if (request.method !== "POST") {
      return new Response("Method Not Allowed", { status: 405 });
    }
    if (!opts.applicationId || !publicKeyData) {
      throw new Error("Handler requires applicationId and applicationPublicKey to be set");
    }
    const hopts = opts;
    const interaction = await validateInteraction(publicKeyData, request);
    if (!interaction)
      return new Response("Unauthorized", { status: 401 });
    if (interaction.type === InteractionType.PING) {
      return handlePingInteraction();
    } else if (interaction.type === InteractionType.APPLICATION_COMMAND) {
      return handleApplicationCommandInteraction(interaction, hopts, env, ctx);
    } else if (interaction.type === InteractionType.MESSAGE_COMPONENT) {
      return handleMessageComponentInteraction(interaction, hopts, env, ctx);
    } else if (interaction.type === InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE) {
      return handleAutocompleteInteraction(interaction, hopts, env, ctx);
    }
    return new Response("Bad Request", { status: 400 });
  };
}
export {
  $actionRow,
  $actionRowChild,
  $embed,
  $field,
  $update,
  APIError,
  AllowedMentionsTypes,
  ApplicationCommandOptionType,
  ApplicationCommandType,
  Button,
  ButtonStyle,
  ChannelType,
  ComponentType,
  Embed,
  Field,
  Fragment,
  InteractionResponseType,
  InteractionType,
  Message,
  Option,
  Row,
  Select,
  authorizeResponse,
  bulkEditApplicationCommandPermissions,
  bulkOverwriteGlobalApplicationCommands,
  bulkOverwriteGuildApplicationCommands,
  call,
  createElement,
  createFollowupMessage,
  createGlobalApplicationCommand,
  createGuildApplicationCommand,
  createHandler,
  deleteFollowupMessage,
  deleteGlobalApplicationCommand,
  deleteGuildApplicationCommand,
  deleteOriginalInteractionResponse,
  deployCommands,
  editApplicationCommandPermissions,
  editFollowupMessage,
  editGlobalApplicationCommand,
  editGuildApplicationCommand,
  editOriginalInteractionResponse,
  getApplicationCommandPermissions,
  getBearerAuth,
  getFollowupMessage,
  getGlobalApplicationCommand,
  getGlobalApplicationCommands,
  getGuildApplicationCommand,
  getGuildApplicationCommandPermissions,
  getGuildApplicationCommands,
  getOriginalInteractionResponse,
  handleLanding,
  normaliseChoices,
  useBoolean,
  useButton,
  useChannel,
  useDefaultPermission,
  useDescription,
  useInteger,
  useMentionable,
  useNumber,
  useRole,
  useSelectMenu,
  useString,
  useUser
};
//# sourceMappingURL=index.js.map
